Operational Control with iCub
=============================


# Prerequisites
By now, you should be an expert of the following components :wink::
- [Joint Control](https://robotology.github.io/robotology-documentation/doc/html/icub_motor_control_tutorial.html).
- [Cartesian Control](https://robotology.github.io/robotology-documentation/doc/html/icub_cartesian_interface.html).
- [Gaze Control](https://robotology.github.io/robotology-documentation/doc/html/icub_gaze_interface.html).

# Assignment
We want you to develop a module that employs the `Joint Interface`, the `Cartesian Interface`
and the `Gaze Interface` to accomplish the following tasks:

1. Make iCub **look down at the table**.
1. **Detect the object** lying on the table.
1. Retrieve the **object position** in the Cartesian domain.
1. Let iCub **look at the object**.
1. Based on the object position, **select the best hand**.
1. **Control the iCub fingers** to achieve suitable grasp configurations.
1. Let iCub **approach the object** to enable a top grasp.
1. Ask iCub to **grasp the object**.
1. **Lift the object**.

The outcome should look like the animation below with the simulator:

<p align="center">
  <img src="assets/grasp-it.gif"/>
</p>

---

Some of the points reported above have been already addressed in the code (e.g. [**object detection and location retrieval**](./src/main.cpp#L38)),
so you need to **fill in the missing gaps** highlighted by the comment `// FILL IN THE CODE` in the [**`src/main.cpp`**](./src/main.cpp) module.

Once done, you can test your code in two ways along with the simulator:

1. **Manually**: running the _yarpmanager scripts_ provided from within [**app/scripts**](./app/scripts) and yielding corresponding commands to the _module rpc port_.
1. **Automatically**: running the script **test.sh** in the **smoke-test** directory. Take into account these important points:
    1. We use a **timeout of _240 seconds_** to check the status of rpc communication, meaning that you have _240 seconds_ max to accomplish each rpc command.
    1. When you reply to rpc commands, we assume the robot has **finished the movement**.
    1. The smoke-test will add a random displacement to the initial position of the ball in order to force the use of both hands.

